
#This code generates one data set from Setting 1 in Supplementary Material.
#Then, it conducts RDMDP and summarizes using mode clustering, credible ball and last iteration.
#We acknowledge that many parts of this code is adapted of PDPMix from R package 'ChiRP'. 
#For more details, see website, https://github.com/stablemarkets/ChiRP developed by Arman Oganisian.

source("~/source_code.R")
library(daarem)
library(fungible)

## Step 1: Generate random covariates (v1, v2, v3) and their classes (c1, c2, c3)
iii=1
set.seed(iii)
XXsim <- monte(seed=123, nvar = 3, nclus = 3, clus.size = c(300,300,300), eta2 = c(0.9, 0.7, 0.7), cor.list=NULL, random.cor = FALSE, skew.list = NULL, kurt.list = NULL, secor = NULL, compactness = NULL, sortMeans = TRUE)
a <- as.data.frame(XXsim$data)
colnames(a) <- c("class", paste0("V", 1:3))
  
## Step 2: Generate random class assignment 
## (2-1) make a 30x30 grid 
library(MASS)
n.col = n.row = 30
cls_mat <- matrix(NA, n.row, n.col, byrow=TRUE)       
mat <- matrix(seq(n.col * n.row), n.row, n.col, byrow=TRUE)
address <- expand.grid(x=1:n.row, y=1:n.col)
z <- rbind(c(-1,0,1,-1,1,-1,0,1),c(-1,-1,-1,0,0,1,1,1)) 
get.neighbors1 <- function(rw){
    z2 <- t(z + unlist(rw)) 
    b.good <- rowSums(z2 > 0) == 2 & z2[,1] <= nrow(cls_mat) & z2[,2] <=ncol(cls_mat) 
    cls_mat[z2[b.good,]]
}
  
## (2-2) assign clustering using adjacency
## (2-2.1) assign a class of element[1,1]  
set.seed(97)                                    
p1 <- runif(n=1, min=0, max=1)
cls_mat[1,1] <- as.numeric(cut(p1, breaks=c(0, 0.33, 0.66, 1))) 
  
## (2-2.2) assign the remaining pixels using this rule
k = 2
aa = c(mat[1:k, 1:k]); bb = c(mat[1:(k-1), 1:(k-1)]); cc = aa[!(aa %in% bb)]
aaa = c(cls_mat[1:(k-1), 1:(k-1)])         
for (i in 1:length(cc)){
    xx = address[cc[i],]$x  
    yy = address[cc[i],]$y  
    p1 = rbinom(n=1, size=1, pr=0.95)          
    existing = c(1,2,3)[c(1,2,3) %in% aaa]
    remaining = c(1,2,3)[!(c(1,2,3) %in% aaa)]
    if (p1 == 1) { 
      cls_mat[xx, yy] <- existing
    } else { 
      b01 <- rbinom(n=1, size=1, pr=0.5)         
      cls_mat[xx, yy] <- remaining[b01 + 1]
    }
  }
  
for(k in 3:ncol(cls_mat)){
    # out-neighbor of k x k  
    aa = c(mat[1:k, 1:k])
    bb = c(mat[1:(k-1), 1:(k-1)])
    cc = aa[!(aa %in% bb)]
    for (i in 1:length(cc)){
      xx = address[cc[i],]$x  
      yy = address[cc[i],]$y  
      p1 = rbinom(n=1, size=1, pr=0.95)          
      existing = c(1,2,3)[c(1,2,3) %in% apply(address, 1, get.neighbors1)[[cc[i]]]]
      remaining = c(1,2,3)[!(c(1,2,3) %in% apply(address, 1, get.neighbors1)[[cc[i]]])]
      if (p1 == 1){                              
        if (length(existing)==2) { 
          b01 <- rbinom(n=1, size=1, pr=0.5)
          cls_mat[xx, yy] <- existing[b01 + 1]
        } else if (length(existing)==3) { 
          n012 <- runif(n=1, min=0, max=1)
          cls_mat[xx, yy] <- existing[as.numeric(cut(n012,breaks=c(0,0.33,0.66,1)))]
        } else if (length(existing)==1){
          cls_mat[xx, yy] <- existing 
        } else {
          n012 <- runif(n=1, min=0, max=1)
          cls_mat[xx, yy] <- remaining[as.numeric(cut(n012,breaks=c(0,0.33,0.66,1)))]
        }
      } else {                                  
        if (length(remaining)==2){
          b01 <- rbinom(n=1, size=1, pr=0.5)
          cls_mat[xx, yy] <- remaining[b01 + 1]
        } else if (length(remaining)==3) { 
          n012 <- runif(n=1, min=0, max=1)
          cls_mat[xx, yy] <- remaining[as.numeric(cut(n012,breaks=c(0,0.33,0.66,1)))]
        } else if (length(remaining)==1) {
          cls_mat[xx, yy] <- remaining
        } else {
          n012 <- runif(n=1, min=0, max=1)
          cls_mat[xx, yy] <- existing[as.numeric(cut(n012,breaks=c(0,0.33,0.66,1)))]
        }
      }
    }
  }
  
## Step 3: Assign V1, V2, and V3 for each grid 
class_ind <- c(cls_mat) 
class_ind <- data.frame(class_ind)
a1 <- a[a$class==1,-1];a2 <- a[a$class==2,-1];a3 <- a[a$class==3,-1]
  
set.seed(3)
for(i in 1:nrow(class_ind)){
    cls <- class_ind[i,1]
    if(cls==1){
      class_ind[i,2:4] <- a1[sample(1:300, size=1, replace=FALSE),1:3]
    } else if (cls==2) {
      class_ind[i,2:4] <- a2[sample(1:300, size=1, replace=FALSE),1:3]
    } else {
      class_ind[i,2:4] <- a3[sample(1:300, size=1, replace=FALSE),1:3]
    }
}
  
df = class_ind
  
## Step 4: Generate binary z with ProbitSimulate
df <- as.matrix(df)
npars <- 3
set.seed(111)
true.beta1 <- c(0.8, 2.5, -1.3)
y1 <- as.matrix(ProbitSimulate(true.beta1, df[df[,1]==1,-1]), ncol=1) 
set.seed(222)
true.beta2 <- c(-1.4, 1.2, 0.8)
y2 <- as.matrix(ProbitSimulate(true.beta2, df[df[,1]==2,-1]), ncol=1)
table(y2)
set.seed(333)
true.beta3 <- rnorm(3, 1, 0.5)  
y3 <- as.matrix(1-ProbitSimulate(true.beta3, df[df[,1]==3,-1]),ncol=1)
  
yy <- c(y1, y2, y3)
df <- as.data.frame(df)
df[,5] <- yy
colnames(df)[5] <- "z"
  
## Step 5: Add location information 
df[,6] <- address$x; df[,7] <- address$y
df <- df[,c(6,7,1:5)]
colnames(df)[c(1,2)] <- c("x", "y")


## Step 6: Split data
set.seed(100)
test_ids <- sample(1:dim(df)[1], round(0.2*dim(df)[1]), F)
d_test <- df[test_ids, ]
d_train <- df[-test_ids,]


## Step 7: Distance matrix
dmat <- dist(df[,c(4:7)], method="euclidean", diag = FALSE)
dmat <- as.matrix(dmat)
dmat01 <- as.matrix(dmat)
dmat01[dmat > sqrt(2)] <- 0   
dmat01[dmat <= sqrt(2)] <- 1    

## Step 8: Run RDMDP 
K = 3
formula =  factor(z) ~ V1 + V2 + V3
fit_0 <- ddPDPMix_noBP_adj(d_train = df[,-c(1:3)], d_test = NULL, adj_mat = dmat01,
                               formula = formula, burnin = 50, iter = 100, 
                               prop_sigma_b = diag(rep(0.001, K+1)),
                               init_a = 0.1, beta_prior_mean=NULL, beta_prior_var=NULL,
                               init_k = 3, tau_scale = 3, mu_scale = 3) 

## Step 9: Mode clustering
mc <- calc_adjmat_n(fit_0$cluster_inds$train, nclus=3)  # mode clustering
cl1 <- as.numeric(gsub("c","",mc$class_mem[,which.min(mc$err)]))
    
## Step 10: Credible ball  
cmat <- gsub("c","",fit_0$cluster_inds$train)
cmat <- matrix(as.numeric(cmat), ncol=ncol(cmat))
library(mcclust); library(mcclust.ext)
psm = comp.psm(t(cmat))
sim.VI = minVI(psm, t(cmat), method=("avg"), include.greedy = TRUE)
cl2 <- sim.VI$cl 

## Step 11: Last Iteration
cl3 <- fit_0$cluster_inds$train[,50]

